package me.hammerle.snuviscript;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UncheckedIOException;
import me.hammerle.code.ISnuviLogger;
import me.hammerle.code.ScriptUtils;
import me.hammerle.code.SnuviParser;
import me.hammerle.exceptions.PreScriptException;

public class SnuviScript {

	public static void main(String[] args) {
		Executable executable;
		switch (args.length) {
			case 0:
				System.out.println("Please enter script location:");
				executable = loadScript(readLine());
				break;
			case 1:
				executable = loadScript(args[0]);
				break;
			default:
				System.err.println("Illegal number of arguments.");
				return;
		}
		ISnuviLogger logger = createLogger();
		SnuviParser  parser = createParser(logger);
		registerTerminalMethods(logger, parser);
		executeScript(executable, logger, parser);
	}

	private static String readLine() throws UncheckedIOException {
		try (BufferedReader in = new BufferedReader(new InputStreamReader(System.in))) {
			return in.readLine();
		} catch (final IOException e) {
			throw new UncheckedIOException(e);
		}
	}

	private static Executable loadScript(String location) {
		File file = new File(location);
		String name = file.getName();
		String code;
		try (BufferedReader reader = new BufferedReader(new FileReader(file))) {
			code = String.join("\n", reader.lines().toArray(String[]::new));
		} catch (final IOException e) {
			throw new UncheckedIOException(e);
		}
		return new Executable(name, code);
	}

	private static ISnuviLogger createLogger() {
		return new FormattedSnuviLogger();
	}

	private static SnuviParser createParser(ISnuviLogger logger) {
		return new SnuviParser(logger, new SnuviSimpleScheduler());
	}

	private static void registerTerminalMethods(ISnuviLogger logger, SnuviParser parser) {
		parser.registerConsumer("print", (o, sc) -> System.out.println(ScriptUtils.connect(o, " ", 0)));
		parser.registerFunction("read", (o, sc) -> readLine());
		parser.registerConsumer("log", (o, sc) -> logger.printInfo(ScriptUtils.connect(o, " ", 0)));
	}

	private static void executeScript(Executable executable, ISnuviLogger logger, SnuviParser parser) {
		try {
			parser.startScript(executable.getName(), executable.getCode());
		} catch (PreScriptException ex) {
			logger.printException(ex, "none", executable.getName(), ex.getLine());
		}
	}
}
